(defvar hh-mode-hook nil)

(defvar hh-keywords
  '("contract" "state" "message"
    "switch receive"
    "while" "if" "then" "else")
  "HH keywords.")

(defvar hh-builtins
  '("new" "dispose" "send" "receive" "open" "close")
  "HH builtins.")

(defvar hh-types
  '("initial" "final" "local")
  "HH types.")

(defvar hh-constants
  '("src" "NULL" "emp")
  "HH constants.")

(defvar hh-keywords-regexp (regexp-opt hh-keywords 'words))
(defvar hh-builtins-regexp (regexp-opt hh-builtins 'words))
(defvar hh-types-regexp (regexp-opt hh-types 'words))
(defvar hh-constants-regexp (regexp-opt hh-constants 'words))

(setq hh-keywords nil)
(setq hh-builtins nil)
(setq hh-types nil)
(setq hh-constants nil)

(defvar hh-font-lock-keywords
  `(
    ("^\\(\\w+\\)(" 1 font-lock-function-name-face)
    ("[][]" . font-lock-doc-face)
    (,hh-types-regexp . font-lock-type-face)
    (,hh-constants-regexp . font-lock-constant-face)
    (,hh-keywords-regexp . font-lock-keyword-face)
    (,hh-builtins-regexp . font-lock-builtin-face)
    ("||" . font-lock-keyword-face)
    ("\\b[0-9]+\\b" . font-lock-constant-face)
    ("\\bval[0-9]*\\b" . font-lock-constant-face)
   ))

;; define the mode, derived from C mode
(define-derived-mode hh-mode c-mode
  "hh mode"
  "Major mode for editing Heap-Hop files."

  (interactive)
  (kill-all-local-variables)

  (setq major-mode 'hh-mode)
  (setq mode-name "HH")

  ;; borrow c's indentation
  (c-mode)
  (setq indent-tabs-mode nil)
  (setq c-basic-offset 2)
  (c-set-offset 'substatement-open 0)
  (c-set-offset 'block-open 0)
  (c-set-offset 'statement-cont 0)

  ;; modifies C syntax table
  (modify-syntax-entry ?_ "w" hh-mode-syntax-table)
  (set-syntax-table hh-mode-syntax-table)

  ;; code for syntax highlighting
  (setq font-lock-defaults '((hh-font-lock-keywords)))

  ;; clear memory
  (setq hh-keywords-regexp nil)
  (setq hh-types-regexp nil)
  (setq hh-constants-regexp nil)
  (setq hh-builtins-regexp nil)
)
